@@ -31,6 +31,7 @@
 #include <libdivecomputer/serial.h>
 #include <libdivecomputer/bluetooth.h>
 #include <libdivecomputer/irda.h>
+#include <libdivecomputer/usb.h>
 #include <libdivecomputer/usbhid.h>
 
 #include "common.h"
@@ -90,8 +91,13 @@ static const backend_table_t g_backends[] = {
 	{"idive",       DC_FAMILY_DIVESYSTEM_IDIVE,    0x03},
 	{"cochran",     DC_FAMILY_COCHRAN_COMMANDER,   0},
 	{"divecomputereu", DC_FAMILY_TECDIVING_DIVECOMPUTEREU, 0},
-	{"descentmk1",  DC_FAMILY_GARMIN, 0},
+	{"mclean",      DC_FAMILY_MCLEAN_EXTREME,      0},
+	{"lynx",        DC_FAMILY_LIQUIVISION_LYNX,    0},
+
+	// Not merged upstream yet
+	{"descentmk1",  DC_FAMILY_GARMIN,              0},
 	{"cosmiq",      DC_FAMILY_DEEPBLU,             0},
+	{"oceans",	DC_FAMILY_OCEANS_S1,           0},
 };
 
 static const transport_table_t g_transports[] = {
@@ -101,6 +107,8 @@ static const transport_table_t g_transports[] = {
 	{"irda",      DC_TRANSPORT_IRDA},
 	{"bluetooth", DC_TRANSPORT_BLUETOOTH},
 	{"ble",       DC_TRANSPORT_BLE},
+
+	// Not merged upstream yet
 	{"usbstorage",DC_TRANSPORT_USBSTORAGE},
 };
 
@@ -399,6 +407,41 @@ dctool_file_read (const char *filename)
 	return buffer;
 }
 
+static dc_status_t
+dctool_usb_open (dc_iostream_t **out, dc_context_t *context, dc_descriptor_t *descriptor)
+{
+	dc_status_t status = DC_STATUS_SUCCESS;
+	dc_iostream_t *iostream = NULL;
+
+	// Discover the usb device.
+	dc_iterator_t *iterator = NULL;
+	dc_usb_device_t *device = NULL;
+	dc_usb_iterator_new (&iterator, context, descriptor);
+	while (dc_iterator_next (iterator, &device) == DC_STATUS_SUCCESS) {
+		break;
+	}
+	dc_iterator_free (iterator);
+
+	if (device == NULL) {
+		ERROR ("No dive computer found.");
+		status = DC_STATUS_NODEVICE;
+		goto cleanup;
+	}
+
+	// Open the usb device.
+	status = dc_usb_open (&iostream, context, device);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR ("Failed to open the usb device.");
+		goto cleanup;
+	}
+
+	*out = iostream;
+
+cleanup:
+	dc_usb_device_free (device);
+	return status;
+}
+
 static dc_status_t
 dctool_usbhid_open (dc_iostream_t **out, dc_context_t *context, dc_descriptor_t *descriptor)
 {
@@ -533,16 +576,18 @@ dctool_iostream_open (dc_iostream_t **iostream, dc_context_t *context, dc_descri
 	case DC_TRANSPORT_SERIAL:
 		return dc_serial_open (iostream, context, devname);
 	case DC_TRANSPORT_USB:
-		return DC_STATUS_SUCCESS;
+		return dctool_usb_open(iostream, context, descriptor);
 	case DC_TRANSPORT_USBHID:
 		return dctool_usbhid_open(iostream, context, descriptor);
 	case DC_TRANSPORT_IRDA:
 		return dctool_irda_open (iostream, context, descriptor, devname);
 	case DC_TRANSPORT_BLUETOOTH:
 		return dctool_bluetooth_open (iostream, context, descriptor, devname);
-	case DC_TRANSPORT_USBSTORAGE:
-		return dc_usb_storage_open (iostream, context, devname);
 	default:
 		return DC_STATUS_UNSUPPORTED;
+
+	// Not merged upstream yet
+	case DC_TRANSPORT_USBSTORAGE:
+		return dc_usb_storage_open (iostream, context, devname);
 	}
 }
