@@ -32,7 +32,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 dc_status_t
-garmin_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream);
+garmin_device_open (dc_device_t **device, dc_context_t *context, dc_iostream_t *iostream, unsigned int model);
 
 dc_status_t
 garmin_parser_create (dc_parser_t **parser, dc_context_t *context);
@@ -49,8 +49,9 @@ garmin_parser_is_dive (dc_parser_t *abstract, const unsigned char *data, unsigne
 // special fixed header in the parser data too.
 #define FIT_NAME_SIZE 24
 
-struct fit_name {
+struct fit_file {
 	char name[FIT_NAME_SIZE + 1];
+	unsigned int mtp_id;
 };
 
 #ifdef __cplusplus
