@@ -74,19 +74,6 @@ static const oceanic_common_device_vtable_t oceanic_vtpro_device_vtable = {
 	oceanic_common_device_profile,
 };
 
-static const oceanic_common_version_t oceanic_vtpro_version[] = {
-	{"VERSAPRO \0\0 256K"},
-	{"ATMOSTWO \0\0 256K"},
-	{"PROPLUS2 \0\0 256K"},
-	{"ATMOSAIR \0\0 256K"},
-	{"VTPRO  r\0\0  256K"},
-	{"ELITE  r\0\0  256K"},
-};
-
-static const oceanic_common_version_t oceanic_wisdom_version[] = {
-	{"WISDOM r\0\0  256K"},
-};
-
 static const oceanic_common_layout_t oceanic_vtpro_layout = {
 	0x8000, /* memsize */
 	0, /* highmem */
@@ -132,6 +119,17 @@ static const oceanic_common_layout_t aeris_500ai_layout = {
 	2, /* pt_mode_serial */
 };
 
+static const oceanic_common_version_t versions[] = {
+	{"VERSAPRO \0\0 256K", 0, &oceanic_vtpro_layout},
+	{"ATMOSTWO \0\0 256K", 0, &oceanic_vtpro_layout},
+	{"PROPLUS2 \0\0 256K", 0, &oceanic_vtpro_layout},
+	{"ATMOSAIR \0\0 256K", 0, &oceanic_vtpro_layout},
+	{"VTPRO  r\0\0  256K", 0, &oceanic_vtpro_layout},
+	{"ELITE  r\0\0  256K", 0, &oceanic_vtpro_layout},
+
+	{"WISDOM r\0\0  256K", 0, &oceanic_wisdom_layout},
+};
+
 static dc_status_t
 oceanic_vtpro_send (oceanic_vtpro_device_t *device, const unsigned char command[], unsigned int csize)
 {
@@ -442,6 +440,20 @@ oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, dc_iostream
 		goto error_free;
 	}
 
+	// Clear the RTS line to reset the PIC inside the data cable as it
+	// may not have have been previously cleared. This ensures that the
+	// PIC will always start in a known state once RTS is set. Starting
+	// in a known default state is very important as the PIC won't
+	// respond to init commands unless it is in a default state.
+	status = dc_iostream_set_rts (device->iostream, 0);
+	if (status != DC_STATUS_SUCCESS) {
+		ERROR (context, "Failed to clear the RTS line.");
+		goto error_free;
+	}
+
+	// Hold RTS clear for a bit to allow PIC to reset.
+	dc_iostream_sleep (device->iostream, 100);
+
 	// Set the RTS line.
 	status = dc_iostream_set_rts (device->iostream, 1);
 	if (status != DC_STATUS_SUCCESS) {
@@ -477,16 +489,15 @@ oceanic_vtpro_device_open (dc_device_t **out, dc_context_t *context, dc_iostream
 		goto error_free;
 	}
 
-	// Override the base class values.
+	// Detect the memory layout.
 	if (model == AERIS500AI) {
 		device->base.layout = &aeris_500ai_layout;
-	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_wisdom_version)) {
-		device->base.layout = &oceanic_wisdom_layout;
-	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_vtpro_version)) {
-		device->base.layout = &oceanic_vtpro_layout;
 	} else {
-		WARNING (context, "Unsupported device detected!");
-		device->base.layout = &oceanic_vtpro_layout;
+		device->base.layout = OCEANIC_COMMON_MATCH(device->base.version, versions, &device->base.firmware);
+		if (device->base.layout == NULL) {
+			WARNING (context, "Unsupported device detected!");
+			device->base.layout = &oceanic_vtpro_layout;
+		}
 	}
 
 	*out = (dc_device_t*) device;
