@@ -21,9 +21,9 @@
 
 #include <stdlib.h>
 
-#include <libdivecomputer/oceanic_vtpro.h>
 #include <libdivecomputer/units.h>
 
+#include "oceanic_vtpro.h"
 #include "oceanic_common.h"
 #include "context-private.h"
 #include "parser-private.h"
@@ -61,14 +61,7 @@ static const dc_parser_vtable_t oceanic_vtpro_parser_vtable = {
 
 
 dc_status_t
-oceanic_vtpro_parser_create (dc_parser_t **out, dc_context_t *context)
-{
-	return oceanic_vtpro_parser_create2 (out, context, 0);
-}
-
-
-dc_status_t
-oceanic_vtpro_parser_create2 (dc_parser_t **out, dc_context_t *context, unsigned int model)
+oceanic_vtpro_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
 	oceanic_vtpro_parser_t *parser = NULL;
 
@@ -136,12 +129,13 @@ oceanic_vtpro_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 			else
 				datetime->year = bcd2dec (((p[32 + 3] & 0xC0) >> 2) + ((p[32 + 2] & 0xF0) >> 4)) + 2000;
 			datetime->month  = (p[4] & 0xF0) >> 4;
-			datetime->day    = bcd2dec (p[3]);
+			datetime->day    = bcd2dec (p[3] & 0x7F);
 			datetime->hour   = bcd2dec (p[1] & 0x7F);
 			pm = p[1] & 0x80;
 		}
 		datetime->minute = bcd2dec (p[0]);
 		datetime->second = 0;
+		datetime->timezone = DC_TIMEZONE_NONE;
 
 		// Convert to a 24-hour clock.
 		datetime->hour %= 12;
