@@ -50,6 +50,7 @@
 #define CHROMIS           0x24
 #define ALADINA1          0x25
 #define MANTIS2           0x26
+#define ALADINA2          0x28
 #define G2                0x32
 #define G2HUD             0x42
 
@@ -532,7 +533,7 @@ uwatec_smart_parser_cache (uwatec_smart_parser_t *parser)
 					parser->model == CHROMIS || parser->model == MANTIS2 ||
 					parser->model == G2 || parser->model == ALADINSPORTMATRIX ||
 					parser->model == ALADINSQUARE || parser->model == G2HUD ||
-					parser->model == ALADINA1) {
+					parser->model == ALADINA1 || parser->model == ALADINA2 ) {
 					unsigned int offset = header->tankpressure + 2 * i;
 					endpressure   = array_uint16_le(data + offset);
 					beginpressure = array_uint16_le(data + offset + 2 * header->ngases);
@@ -626,6 +627,7 @@ uwatec_smart_parser_create (dc_parser_t **out, dc_context_t *context, unsigned i
 	case G2HUD:
 	case ALADINSPORTMATRIX:
 	case ALADINA1:
+	case ALADINA2:
 		parser->headersize = 84;
 		parser->header = &uwatec_smart_trimix_header;
 		parser->samples = uwatec_smart_galileo_samples;
@@ -969,7 +971,7 @@ uwatec_smart_parse (uwatec_smart_parser_t *parser, dc_sample_callback_t callback
 			parser->model == CHROMIS || parser->model == MANTIS2 ||
 			parser->model == G2 || parser->model == ALADINSPORTMATRIX ||
 			parser->model == ALADINSQUARE || parser->model == G2HUD ||
-			parser->model == ALADINA1) {
+			parser->model == ALADINA1 || parser->model == ALADINA2 ) {
 			// Uwatec Galileo
 			id = uwatec_galileo_identify (data[offset]);
 		} else {
